const { Channel } = require('twlv-swarm');
const http = require('http');
const ioclient = require('socket.io-client');
const ioserver = require('socket.io');
const { URL } = require('url');
const { EventEmitter } = require('events');

class SocketIo extends Channel {
  constructor ({ port, server, listening = true }) {
    super('socketio');

    this.listening = listening;
    if (this.listening) {
      this.port = port;
      this.server = server;
      this.localServer = Boolean(!server);
    }
  }

  async up () {
    if (!this.listening) {
      return;
    }

    if (this.localServer) {
      this.server = http.Server((req, res) => {
        res.end(JSON.stringify({ name: 'twlv-channel-socketio' }));
      });
    }

    let io = ioserver(this.server);
    io.on('connection', ioSocket => {
      let socket = this.ioSocket2Socket(ioSocket);
      this.incoming(socket);
    });

    if (this.localServer) {
      await new Promise(resolve => {
        this.server.listen(this.port, resolve);
      });
    }
  }

  down () {
    if (!this.listening) {
      return;
    }

    if (this.localServer) {
      this.server.close();
    }
  }

  onMessage () {
    // noop
  }

  async connect (url) {
    let ioSocket = ioclient(this.p2p2HttpUrl(url));

    await new Promise((resolve, reject) => {
      ioSocket.on('connect', resolve);
    });

    let socket = this.ioSocket2Socket(ioSocket);
    return socket;
  }

  p2p2HttpUrl (url) {
    let urlO = new URL(url);
    return `http://${urlO.host}`;
  }

  ioSocket2Socket (ioSocket) {
    return new Socket(ioSocket);
  }

  formatUrl (ip) {
    if (!this.listening) {
      return;
    }

    return `${this.proto}://${ip}:${this.port}`;
  }

  dump () {
    let { proto, port } = this;
    return { proto, port };
  }
}

class Socket extends EventEmitter {
  constructor (io) {
    super();

    this.io = io;
    this.io.on('message', message => {
      // debug(`w|${message}|`);
      if (this.stream) {
        this.stream.write(message);
      } else {
        this.emit('data', message);
      }
    });
  }

  pipe (stream) {
    this.stream = stream;
    return stream;
  }

  write (chunk) {
    // debug(`s|${chunk}|`);
    this.io.send(chunk);
  }

  end () {
    this.io.disconnect();
  }
}

module.exports = SocketIo;
