const { Swarm } = require('twlv-swarm');
const { Logger } = require('twlv-logger');
const SocketIo = require('../');
const assert = require('assert');
const logger = new Logger('test/connect-each-other');
const Mock = require('twlv-swarm/app/mock');

describe('Case: connect each other', () => {
  beforeEach(() => {
    process.on('unhandledRejection', err => console.error('Unhandled rejection', err));
  });

  afterEach(async () => {
    process.removeAllListeners('unhandledRejection');
    await reset();
  });

  it('connected to each other', async () => {
    let swarm1 = createSwarm(12001);
    let swarm2 = createSwarm(12002, 'socketio://localhost:12001');

    await swarm1.start();
    await swarm2.start();

    logger.log('Swarms started');

    assert.equal(swarm2.peers.length, 1);

    let message = Mock.createMessage(swarm1.address, 'foo', 'bar');
    await swarm2.send(message);
    await swarm1.getApp('mock').waitForMessage(message);
  });

  let swarms = [];
  function createSwarm (port, peer) {
    let channel = new SocketIo({ port });
    let swarm = new Swarm();
    swarm.addChannel(channel);

    if (peer) {
      swarm.addBootPeer(peer);
    }

    swarm.addApp(new Mock());

    swarms.push(swarm);
    return swarm;
  }

  async function reset () {
    await Promise.all(swarms.map(swarm => swarm.stop()));

    swarms = [];
  }
});
