const { Swarm } = require('twlv-swarm');
const { Logger } = require('twlv-logger');
const SocketIo = require('../');
const Tcp = require('twlv-swarm/channel/tcp');
const PeerLookup = require('twlv-swarm/discovery/peer-lookup');
const Mock = require('twlv-swarm/app/mock');
const assert = require('assert');
const logger = new Logger('test/connect-other-channel');
const sleep = require('es7-sleep');

describe('Case: connect other channel', () => {
  beforeEach(() => {
    process.on('unhandledRejection', err => console.error('Unhandled rejection', err));
  });

  afterEach(async () => {
    process.removeAllListeners('unhandledRejection');
    await reset();
  });

  it('connected to other channel', async () => {
    let swarm1 = createSwarm({ socketio: 12001, tcp: 12121 });
    let swarm2 = createSwarm({ socketio: 12002 }, 'socketio://localhost:12001');
    let swarm3 = createSwarm({ tcp: 12122 }, 'tcp://localhost:12121');

    await swarm1.start();
    await swarm2.start();
    await swarm3.start();

    await sleep(10);

    logger.log('Swarms started');
    logger.log('Swarm1:', swarm1.address);
    logger.log('Swarm2:', swarm2.address);
    logger.log('Swarm3:', swarm3.address);

    let message = Mock.createMessage(swarm2.address, 'foo', 'bar');
    await swarm3.send(message);
    await swarm2.getApp('mock').waitForMessage(message);
  });

  let swarms = [];
  function createSwarm (ports, peer) {
    let swarm = new Swarm();

    for (let key in ports) {
      let port = ports[key];
      if (key === 'socketio') {
        swarm.addChannel(new SocketIo({ port }));
      } else if (key === 'tcp') {
        swarm.addChannel(new Tcp({ port }));
      }
    }

    if (peer) {
      swarm.addBootPeer(peer);
    }

    swarm.addApp(new Mock());
    swarm.addDiscovery(new PeerLookup());

    swarms.push(swarm);
    return swarm;
  }

  async function reset () {
    await Promise.all(swarms.map(swarm => swarm.stop()));
    swarms = [];
  }
});
