const { Swarm } = require('twlv-swarm');
const logger = require('twlv-logger')('twlv-channel-socketio:test:half-capability');
const Socketio = require('../');
const PeerLookup = require('twlv-swarm/discovery/peer-lookup');
const assert = require('assert');

describe('half capability', () => {
  it('use non-listening channel', async () => {
    let swarm = createSwarm({ listening: false });

    await swarm.start();
    assert.equal(swarm.advertisement.urls.length, 0);
  });

  it('send message to non-listening', async () => {
    let relay = createSwarm({ port: 12000 });
    let swarm1 = createSwarm({ listening: false });
    let swarm2 = createSwarm({ listening: false });

    await relay.start();
    await swarm1.start();
    await swarm2.start();

    await swarm1.connect('socketio://localhost:12000');
    await swarm2.connect('socketio://localhost:12000');

    let peer = await swarm1.lookup(swarm2.address);
    assert.equal(peer.urls.length, 0);
  });

  let swarms = [];
  function createSwarm (options) {
    let swarm = new Swarm();
    swarm.addChannel(new Socketio(options));
    swarm.addDiscovery(new PeerLookup());
    swarms.push(swarm);
    return swarm;
  }

  afterEach(async () => {
    await Promise.all(swarms.map(swarm => swarm.stop()));
    swarms = [];
  });
});
